Package: genoPlotR
Type: Package
Title: Plot Publication-Grade Gene and Genome Maps
Version: 0.8.11
Date: 2021-01-05
Author: Lionel Guy <lionel.guy@imbim.uu.se>
URL: http://genoplotr.r-forge.r-project.org/
Depends: R (>= 2.10.0), ade4, grid
Imports: methods
Maintainer: Lionel Guy <lionel.guy@imbim.uu.se>
Description: Draws gene or genome maps and comparisons between these, in a 
   publication-grade manner. Starting from simple, common files, it will 
   draw postscript or PDF files that can be sent as such to journals.
License: GPL (>= 2)
LazyLoad: yes
NeedsCompilation: no
Packaged: 2021-01-07 13:23:56 UTC; lionel
Repository: CRAN
Date/Publication: 2021-01-07 15:00:02 UTC
Built: R 4.6.0; ; 2026-04-27 09:39:41 UTC; unix
